! initial version at 02/07/2008
! Equations, parameters, etc are based on the paper: M. S. Baptista et. al, PRE, 82, 036203, 2010
! 1st revision at 17/05/2013
! 2nd revision at 26/05/2013
! 3rd revision at 31/05/2013
! 4th revision at 01/06/2013
! 5th revision at 08/06/2013
! 6th revision at 10/06/2013
! 7th revision at 26/06/2013
! 8th revision at 27/06/2013
! 9th revision at 03/07/2013
! 10th revision at 29/07/2013
! 11th revision at 27/09/2013
! 12th revision at 06/11/2013

! matlab -nodisplay -desktop -r "plot_neural_network('n20c4t2',4,5),quit"


program nne_v12
implicit none

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!dimensao fisica da matriz == NUMBER OF NEURONS
integer(kind=4)			:: num_neurons
parameter(num_neurons=277)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!NUMBER OF COMMUNITIES
integer(kind=4)			:: num_communities

parameter(num_communities=6)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!number of Lyapunov exponents
integer(kind=4)			:: num_lyapu=2 !BE CAREFUL: it can not be > num_neurons * system_dim
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!if syncmatrixglobalnetwork_switch=1 then computation of the synchronization matrix of the whole network will be computed and plotted
!if syncmatrixglobalnetwork_switch/=1 then computation of the synchronization matrix of the whole network will NOT be computed and plotted
integer(kind=4)			:: syncmatrixglobalnetwork_switch
parameter(syncmatrixglobalnetwork_switch=1)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!param_chem_coupling is the parameter for the chemical (non linear) coupling
!param_elec_coupling is the parameter for the electrical (linear) coupling
real(kind=8)			:: param_chem_coupling,param_elec_coupling
! !variation of the chemical coupling
! parameter(param_chem_coupling=4.d0/(1.d0*(num_neurons-1))) !---> to avoid complete synchronization
! !variation of electrical coupling ---> =0.5/10 ---> to avoid complete synchronization
! parameter(param_elec_coupling=0.4d0)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!kind of Runge Kutta method for the computation of the trajectory
!if RK_method_index_traj=1 then 1-stage Euler method
!if RK_method_index_traj=2 then 2-stage Heun method (or improved Euler method)
!if RK_method_index_traj=3 then 2-stage Arithmetic Mean Runge - Kutta method (3rd order)
!if RK_method_index_traj=4 then classical Runge - Kutta method (4th order)
integer(kind=4)			:: RK_method_index_traj
parameter(RK_method_index_traj=1)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!kind of Runge Kutta method for the computation of the Lyapunov exponents
!if RK_method_index_LEs=1 then 1-stage Euler method
!if RK_method_index_LEs=2 then 2-stage Heun method (or improved Euler method)
!if RK_method_index_LEs=3 then 2-stage Arithmetic Mean Runge - Kutta method (3rd order)
!if RK_method_index_LEs=4 then classical Runge - Kutta method (4th order)
integer(kind=4)			:: RK_method_index_LEs
parameter(RK_method_index_LEs=RK_method_index_traj)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!system_dim is the number of variables of the solution -----> p,q,n,phase
integer(kind=4)			:: system_dim,dim_par
parameter(system_dim=4) !DO NOT CHANGE THIS VALUE!
parameter(dim_par=6) !DO NOT CHANGE THIS VALUE!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!quantas integracoes para cada Gram–Schmidt ==
!== the Gram-Schmidt process for Lyapunov exponents is performed at multiples of n_step time steps
integer(kind=4)			:: n_step
parameter(n_step=100)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!time step of the Runge-Kutta integrator
real(kind=8)			:: dt
parameter(dt=0.01d0)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
real(kind=8)			:: t0
parameter(t0=0.d0) !DO NOT CHANGE THIS VALUE!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!transiente do atrator continuo
integer(kind=4)			:: trans_traj
parameter(trans_traj=30000)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!quant. de pontos na sessao "estroboscopica" with tam_traj>trans_traj
integer(kind=4)			:: tam_traj
parameter(tam_traj=500000)


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
real(kind=8)			:: I_ext
parameter(I_ext=3.25d0)
!if I_ext_switch=0 then I_ex=3.25 ---> for this value the system exhibits multi-time-scale chaotic behavior characterized as spike bursting
!if I_ext_switch=1 then I_ext changes in time in a sinusoidal way with a small random perturbation
integer(kind=4)			:: I_ext_switch
parameter(I_ext_switch=0)
real(kind=8)			:: I_ext_period,pi,I_ext_amplitude
parameter(pi=dacos(-1.d0))
parameter(I_ext_amplitude=0.3d0)
parameter(I_ext_period=pi/(2.d0*(tam_traj-trans_traj)*dt))
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
real(kind=8)			:: r,Theta_syn
parameter(r=0.005d0) !DO NOT CHANGE THIS VALUE!
parameter(Theta_syn=-0.25d0) !DO NOT CHANGE THIS VALUE!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!if V_syn=2 excitatory behavior
!if V_syn=-2 inhibitory behavior
real(kind=8)			:: V_syn
parameter(V_syn=2.d0) !DO NOT CHANGE THIS VALUE!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!amplitude variation on the initial conditions of the neurons
real(kind=8)			:: mult_ci
parameter(mult_ci=0.5d0)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!if write_traj_file_switch=0 the trajectory is NOT written in the file,otherwise it is
integer(kind=4)			:: write_traj_file_switch
parameter(write_traj_file_switch=0)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!if write_LEs_file_switch=0 the Lyapunov exponents are NOT written in the file,otherwise they are
integer(kind=4)			:: write_LEs_file_switch
parameter(write_LEs_file_switch=0)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!vezes desprezadas no calculo do Gram–Schmidt
integer(kind=4)			:: lyapu_on
parameter(lyapu_on=1)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
real(kind=8)			:: OP
parameter(OP=-1.d0)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
integer(kind=4)			:: seed_ci_initial
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!escolhe_geometria=1,read from files the adjacency matrices for the chemical and electrical connections
integer(kind=4)			:: escolhe_geometria
parameter(escolhe_geometria=1)
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

real(kind=4)			:: time_begin,time_end
real(kind=8)			:: Ic,order_parameter_r_full_network,rlink,max_LE_final
real(kind=8),allocatable	:: order_parameter_r_communities(:),syncmatrixglobalnetwork(:,:)
integer(kind=4),allocatable	:: matriz_acopla_electrical(:,:),matriz_acopla_chemical(:,:),numcom(:),community(:,:)
integer(kind=4)			:: i,j,counter_realization,counter
character(1)			:: char_i1
character(2)			:: char_i2
character(3)			:: char_i3,char_num_communities
character(4)			:: char_num_neurons,char_counter_realization
character(26)			:: arg_cc,arg_ec,arg_counter_realization,char_param_chem_coupling,char_param_elec_coupling

call cpu_time(time_begin)

!here we read from the command prompt the electrical and chemical potential values
!the old way to read from the command prompt
!call getarg(1,arg_cc)
!call getarg(2,arg_ec)
!call getarg(3,arg_counter_realization)

!the new way to read from the command prompt
call get_command_argument(1,arg_cc)
call get_command_argument(2,arg_ec)
call get_command_argument(3,arg_counter_realization)

read(arg_cc,'(f26.16)') param_chem_coupling
read(arg_ec,'(f26.16)') param_elec_coupling
read(arg_counter_realization,'(i4)') counter_realization

write(char_param_chem_coupling,'(f26.16)') param_chem_coupling
write(char_param_elec_coupling,'(f26.16)') param_elec_coupling
write(char_counter_realization,'(i4)') counter_realization

call systemqq("mkdir -p 'cc="//char_param_chem_coupling//"_ec="//char_param_elec_coupling//"/"//char_counter_realization//"rn'")

call initialize_random_number_seed(seed_ci_initial)

if (num_lyapu<2) num_lyapu=2

!detection and creation of the communities of the network
allocate(numcom(num_communities))
numcom=0
do i=1,num_communities
	if (i<=9) then
		write(char_i1,'(i1)') i
		open(1,file='../../community'//char_i1//'.in',status='old')
		counter=0
		do while (.not.eof(1)) 
			read(1,*) j
			counter=counter+1
		enddo
		numcom(i)=counter
		close(1)
	endif
	if (10<=i.and.i<=99) then
		write(char_i2,'(i2)') i
		open(1,file='../../community'//char_i2//'.in',status='old')
		counter=0
		do while (.not.eof(1)) 
			read(1,*) j
			counter=counter+1
		enddo
		numcom(i)=counter
		close(1)
	endif
	if (100<=i.and.i<=999) then
		write(char_i3,'(i3)') i
		open(1,file='../../community'//char_i3//'.in',status='old')
		counter=0
		do while (.not.eof(1)) 
			read(1,*) j
			counter=counter+1
		enddo
		numcom(i)=counter
		close(1)
	endif
enddo

allocate(community(num_communities,maxval(numcom(:))))
community=0

do i=1,num_communities
	if (i<=9) then
		write(char_i1,'(i1)') i
		open(1,file='../../community'//char_i1//'.in',status='old')
		counter=0
		do while (.not.eof(1)) 
			counter=counter+1
			read(1,*) community(i,counter)
		enddo
		close(1)
	endif
	if (10<=i.and.i<=99) then
		write(char_i2,'(i2)') i
		open(1,file='../../community'//char_i2//'.in',status='old')
		counter=0
		do while (.not.eof(1)) 
			counter=counter+1
			read(1,*) community(i,counter)
		enddo
		close(1)
	endif
	if (100<=i.and.i<=999) then
		write(char_i3,'(i3)') i
		open(1,file='../../community'//char_i3//'.in',status='old')
		counter=0
		do while (.not.eof(1)) 
			counter=counter+1
			read(1,*) community(i,counter)
		enddo
		close(1)
	endif
enddo

allocate(matriz_acopla_electrical(num_neurons,num_neurons),matriz_acopla_chemical(num_neurons,num_neurons))
matriz_acopla_electrical=0
matriz_acopla_chemical=0
if (syncmatrixglobalnetwork_switch==1) allocate(syncmatrixglobalnetwork(num_neurons,num_neurons))
allocate(order_parameter_r_communities(num_communities))
	
!gera parametros e acoplamentos de forma aleatoria gradual
call gera_para_dedo_09(num_neurons,escolhe_geometria,matriz_acopla_electrical,matriz_acopla_chemical,char_param_chem_coupling,char_param_elec_coupling,char_counter_realization)

write(char_num_neurons,'(i4)') num_neurons !number of neurons
write(char_num_communities,'(i3)') num_communities !number of communities
	
open(1,file='cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'info_nnev12n'//char_num_neurons//'.txt',status='replace')
call create_plot_gnu_file(char_param_chem_coupling,char_param_elec_coupling,char_num_neurons,char_counter_realization)
open(21,file='cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'S_global_S_clusters_final_time_nnev12n'//char_num_neurons//'.dat',status='replace')
if (syncmatrixglobalnetwork_switch==1) then
	call create_plot_rlink_glob_synch_average_C_matr_gnu_file(char_param_chem_coupling,char_param_elec_coupling,char_num_neurons,char_counter_realization)
	open(23,file='cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'rlink_glob_synch_average_C_matr_nnev12n'//char_num_neurons//'.dat',status='replace')
endif

call open_various_files(num_lyapu,write_traj_file_switch,write_LEs_file_switch,syncmatrixglobalnetwork_switch,char_param_chem_coupling,char_param_elec_coupling,char_num_neurons,char_counter_realization)

call bif_neuron_v12_mpi(num_neurons,num_lyapu,RK_method_index_traj,RK_method_index_LEs,system_dim,dim_par,n_step,trans_traj,tam_traj,write_traj_file_switch,write_LEs_file_switch,lyapu_on,seed_ci_initial,param_chem_coupling,param_elec_coupling,dt,t0,r,Theta_syn,V_syn,mult_ci,OP,escolhe_geometria,matriz_acopla_electrical,matriz_acopla_chemical,Ic,char_num_neurons,I_ext_switch,I_ext,I_ext_period,I_ext_amplitude,order_parameter_r_full_network,syncmatrixglobalnetwork,rlink,syncmatrixglobalnetwork_switch,char_param_chem_coupling,char_param_elec_coupling,max_LE_final,char_counter_realization,num_communities,numcom,community,order_parameter_r_communities)

if (syncmatrixglobalnetwork_switch==1) then
	write(22,'(<num_neurons>f12.5)') ((syncmatrixglobalnetwork(i,j),j=1,num_neurons),i=1,num_neurons)
	call flush(22)
endif
call close_various_files(num_lyapu,write_traj_file_switch,write_LEs_file_switch,syncmatrixglobalnetwork_switch)
if (syncmatrixglobalnetwork_switch==1) then
	call create_S_global_synchronization_matrix_final_time_plot_gnu_file(num_neurons,char_param_chem_coupling,char_param_elec_coupling,char_num_neurons,char_counter_realization)
	call systemqq('gnuplot < ''cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'plot_S_global_synchronization_matrix_final_time_nnev12n'//char_num_neurons//'.gnu''')
endif

open(8,file='cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'adjacency_matrix_electrical_coupling_nnev12n'//char_num_neurons//'.txt',status='replace')
open(9,file='cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'adjacency_matrix_chemical_coupling_nnev12n'//char_num_neurons//'.txt',status='replace')
open(25,file='cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'total_adjacency_matrix_nnev12n'//char_num_neurons//'.dat',status='replace')
write(8,'(<num_neurons>i3)') ((matriz_acopla_electrical(i,j),i=1,num_neurons),j=1,num_neurons)
call flush(8)
write(9,'(<num_neurons>i3)') ((matriz_acopla_chemical(i,j),i=1,num_neurons),j=1,num_neurons)
call flush(9)
write(25,'(<num_neurons>i3)') ((matriz_acopla_electrical(i,j)+matriz_acopla_chemical(i,j),i=1,num_neurons),j=1,num_neurons)
call flush(25)
close(8)
close(9)
close(25)
call create_plot_total_adjacency_matrix_gnu_file(char_param_chem_coupling,char_param_elec_coupling,char_num_neurons,char_counter_realization)
call systemqq('gnuplot < ''cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'plot_total_adjacency_matrix_nnev12n'//char_num_neurons//'.gnu''')
write(21,'(<1+num_communities>f26.16)') order_parameter_r_full_network,(order_parameter_r_communities(i),i=1,num_communities)
call flush(21)

call create_S_global_S_clusters_vs_time_plot_gnu_file(char_param_chem_coupling,char_param_elec_coupling,char_num_neurons,char_counter_realization,num_communities)

call systemqq('gnuplot < ''cc='//char_param_chem_coupling//'_ec='//char_param_elec_coupling//"/"//char_counter_realization//"rn/"//'plot_S_global_S_clusters_vs_time_nnev12n'//char_num_neurons//'.gnu''')

if (syncmatrixglobalnetwork_switch==1) then
	write(23,'(2f26.16)') rlink,order_parameter_r_full_network
	call flush(23)
endif

deallocate(matriz_acopla_electrical,matriz_acopla_chemical,numcom,community)

deallocate(order_parameter_r_communities)
if (syncmatrixglobalnetwork_switch==1) deallocate(syncmatrixglobalnetwork)

close(21)
if (syncmatrixglobalnetwork_switch==1) close(23)

open(24,file='ec_vs_cc_scanning_nnev12'//char_num_neurons//'.dat.'//char_counter_realization,status='replace')
write(24,'(6f26.16,i4)') param_chem_coupling,param_elec_coupling,Ic,rlink,order_parameter_r_full_network,max_LE_final,counter_realization
call flush(24)
close(24)

call write_used_time(time_begin)

close(1)

end program nne_v12
